#Undef Min
#Undef Max
Type Color_t
	R As Double
	G As Double
	B As Double
	A As Double
End Type
Function Min(a As Double,b As double) As Double
	Return (IIf(a<b,a,b))
End Function
Function Max(a As Double,b As double) As Double
	Return (IIf(a>b,a,b))
End Function
Function Hue_2_RGB( v1 As Double, v2 As Double, vH  As Double) As Double            '//Function Hue_2_RGB
	if ( vH < 0 ) Then vH += 1
	if ( vH > 1 ) Then vH -= 1
	if ( ( 6 * vH ) < 1 ) Then Return ( v1 + ( v2 - v1 ) * 6 * vH )
	if ( ( 2 * vH ) < 1 ) Then Return ( v2 )
	if ( ( 3 * vH ) < 2 ) Then Return ( v1 + ( v2 - v1 ) * ( ( 2 / 3 ) - vH ) * 6 )
	return ( v1 )
End Function
Function RGB2XYZ(R As UByte,G As UByte,B As UByte) As UInteger
	Dim As Double var_R = ( R / 255 )        'R from 0 to 255
	Dim As Double var_G = ( G / 255 )        'G from 0 to 255
	Dim As Double var_B = ( B / 255 )        'B from 0 to 255

	if ( var_R > 0.04045 ) Then
		var_R = ( ( var_R + 0.055 ) / 1.055 ) ^ 2.4
	Else
		var_R = var_R / 12.92
	endif
	if ( var_G > 0.04045 ) Then
		var_G = ( ( var_G + 0.055 ) / 1.055 ) ^ 2.4
	Else
		var_G = var_G / 12.92
	EndIf
	if ( var_B > 0.04045 ) Then
		var_B = ( ( var_B + 0.055 ) / 1.055 ) ^ 2.4
	Else
		var_B = var_B / 12.92
	EndIf
	var_R = var_R * 100
	var_G = var_G * 100
	var_B = var_B * 100

	'Observer. = 2, Illuminant = D65
	Dim As Double X = var_R * 0.4124 + var_G * 0.3576 + var_B * 0.1805
	Dim As Double Y = var_R * 0.2126 + var_G * 0.7152 + var_B * 0.0722
	Dim As Double Z = var_R * 0.0193 + var_G * 0.1192 + var_B * 0.9505

	Dim c As UInteger
	c=&hff000000+(X Shl 16)+(Y Shl 8)+Z
	Return c
End Function
Function XYZ2RGB(X As UByte,Y As UByte,Z As UByte) As UInteger
	Dim As Double var_X = X / 100        'X from 0 to  95.047      (Observer = 2, Illuminant = D65)
	Dim As Double var_Y = Y / 100        'Y from 0 to 100.000
	Dim As Double var_Z = Z / 100        'Z from 0 to 108.883

	Dim As Double var_R = var_X *  3.2406 + var_Y * -1.5372 + var_Z * -0.4986
	Dim As Double var_G = var_X * -0.9689 + var_Y *  1.8758 + var_Z *  0.0415
	Dim As Double var_B = var_X *  0.0557 + var_Y * -0.2040 + var_Z *  1.0570

	if ( var_R > 0.0031308 ) Then
		var_R = 1.055 * ( var_R ^ ( 1 / 2.4 ) ) - 0.055
	Else
		var_R = 12.92 * var_R
	EndIf
	if ( var_G > 0.0031308 ) Then
		var_G = 1.055 * ( var_G ^ ( 1 / 2.4 ) ) - 0.055
	Else
		var_G = 12.92 * var_G
	EndIf
	If ( var_B > 0.0031308 ) Then
		var_B = 1.055 * ( var_B ^ ( 1 / 2.4 ) ) - 0.055
	Else
		var_B = 12.92 * var_B
	EndIf
	Dim As UByte R = Int(var_R * 255)
	Dim As UByte G = Int(var_G * 255)
	Dim As UByte B = Int(var_B * 255)
	
	Dim c As UInteger
	c=&hff000000+(R Shl 16)+(G Shl 8)+B
	Return c
End Function
Function RGB2HSL(R As UByte,G As UByte,B As UByte) As Color_t
	Dim As Double H                                 'HSL results from 0 to 1
	Dim As Double S
	Dim As Double var_R = ( R / 255 )                     'RGB from 0 to 255
	Dim As Double var_G = ( G / 255 )
	Dim As Double var_B = ( B / 255 )

	Dim As Double var_Min = Min( var_R, Min(var_G, var_B))    'Min. value of RGB
	Dim As Double var_Max = Max( var_R, Max(var_G, var_B))    'Max. value of RGB
	Dim As Double del_Max = var_Max - var_Min             'Delta RGB value

	Dim As Double L = ( var_Max + var_Min ) / 2

	if ( del_Max = 0 ) Then                    'This is a gray, no chroma...
		H = 0                                'HSL results from 0 to 1
		S = 0
	Else                                    'Chromatic data...
		if ( L < 0.5 ) Then
			S = del_Max / ( var_Max + var_Min )
		Else
			S = del_Max / ( 2 - var_Max - var_Min )
		EndIf
		Dim As Double del_R = ( ( ( var_Max - var_R ) / 6 ) + ( del_Max / 2 ) ) / del_Max
		Dim As Double del_G = ( ( ( var_Max - var_G ) / 6 ) + ( del_Max / 2 ) ) / del_Max
		Dim As Double del_B = ( ( ( var_Max - var_B ) / 6 ) + ( del_Max / 2 ) ) / del_Max

		if      ( var_R = var_Max ) Then
			H = del_B - del_G
		ElseIf ( var_G = var_Max ) Then
			H = ( 1 / 3 ) + del_R - del_B
		ElseIf ( var_B = var_Max ) Then
			H = ( 2 / 3 ) + del_G - del_R
		EndIf

		if ( H < 0 ) Then H += 1
		if ( H > 1 ) Then H -= 1
	EndIf

	Dim c As Color_t
	c.R=H
	c.G=S
	c.B=L
	Return c
End Function

Function HSL2RGB(H As Double,S As Double,L As Double) As UInteger
	Dim As Double var_1,var_2
	Dim As UByte R,G,B
	if ( S = 0 ) Then                     'HSL from 0 to 1
		R = L * 255                      'RGB results from 0 to 255
		G = L * 255
		B = L * 255
	Else
		if ( L < 0.5 ) Then
			var_2 = L * ( 1 + S )
		Else
			var_2 = ( L + S ) - ( S * L )
		EndIf
		var_1 = 2 * L - var_2

		R = 255 * Hue_2_RGB( var_1, var_2, H + ( 1 / 3 ) )
		G = 255 * Hue_2_RGB( var_1, var_2, H )
		B = 255 * Hue_2_RGB( var_1, var_2, H - ( 1 / 3 ) )
	endif

	Dim c As UInteger
	c=&hff000000+(R Shl 16)+(G Shl 8)+B
	Return c
End Function
Function RGB2HSV(R As UByte,G As UByte,B As UByte) As Color_t
	Dim As Double H,S,V
	Dim As Double var_R = ( R / 255 )                     'RGB from 0 to 255
	Dim As Double var_G = ( G / 255 )
	Dim As Double var_B = ( B / 255 )
	Dim As Double var_Min = Min( var_R, Min(var_G, var_B) )    'Min. value of RGB
	Dim As Double var_Max = Max( var_R, Max(var_G, var_B) )    'Max. value of RGB
	Dim As Double del_Max = var_Max - var_Min             'Delta RGB value
	Dim As Double del_R,del_G,del_B
	V = var_Max

	if ( del_Max = 0 ) Then                    'This is a gray, no chroma...
		H = 0                                'HSV results from 0 to 1
		S = 0
	Else                                    'Chromatic data...
		S = del_Max / var_Max

		del_R = ( ( ( var_Max - var_R ) / 6 ) + ( del_Max / 2 ) ) / del_Max
		del_G = ( ( ( var_Max - var_G ) / 6 ) + ( del_Max / 2 ) ) / del_Max
		del_B = ( ( ( var_Max - var_B ) / 6 ) + ( del_Max / 2 ) ) / del_Max

		if      ( var_R = var_Max ) Then
			H = del_B - del_G
		ElseIf ( var_G = var_Max ) Then
			H = ( 1 / 3 ) + del_R - del_B
		ElseIf ( var_B = var_Max ) Then
			H = ( 2 / 3 ) + del_G - del_R
		EndIf
		if ( H < 0 ) Then H += 1
		if ( H > 1 ) Then H -= 1
	EndIf
	Dim c As Color_t
	c.R=H
	c.G=S
	c.B=V
	Return c
End Function
Function HSV2RGB(H As Double,S As Double,V As Double) As UInteger
	Dim as UByte R,G,B
	Dim As Double var_r,var_g,var_b
	Dim As Double var_h,var_i,var_1,var_2,var_3


	if ( S = 0 ) Then                      'HSV from 0 to 1
		R = V * 255
		G = V * 255
		B = V * 255
	Else
		var_h = H * 6
		if ( var_h = 6 ) Then var_h = 0      'H must be < 1
		var_i = int( var_h )             'Or ... var_i = floor( var_h )
		var_1 = V * ( 1 - S )
		var_2 = V * ( 1 - S * ( var_h - var_i ) )
		var_3 = V * ( 1 - S * ( 1 - ( var_h - var_i ) ) )

		if      ( var_i = 0 ) Then
			var_r = V : var_g = var_3 : var_b = var_1
		ElseIf ( var_i = 1 ) Then
			var_r = var_2 : var_g = V : var_b = var_1
		ElseIf ( var_i = 2 ) Then
			var_r = var_1 : var_g = V : var_b = var_3
		ElseIf ( var_i = 3 ) Then
			var_r = var_1 : var_g = var_2 : var_b = V
		ElseIf ( var_i = 4 ) Then
			var_r = var_3 : var_g = var_1 : var_b = V
		else
			var_r = V : var_g = var_1 : var_b = var_2
		EndIf

		R = var_r * 255                  'RGB results from 0 to 255
		G = var_g * 255
		B = var_b * 255
	endif

	Dim c As UInteger
	c=&hff000000+(R Shl 16)+(G Shl 8)+B
	Return c
End Function
Function RGB2CMY(R As UByte,G As UByte,B As UByte) As Color_t
'RGB values from 0 to 255
'CMY results from 0 to 1
Dim As Double C,M,Y
C = 1 - ( R / 255 )
M = 1 - ( G / 255 )
Y = 1 - ( B / 255 )

	Dim c1 As Color_t
	c1.R=C
	c1.G=M
	c1.B=Y
	Return c1
End Function
Function CMY2RGB(C As Double,M As Double,Y As Double) As UInteger
'CMY values from 0 to 1
'RGB results from 0 to 255
Dim As UByte R,G,B
R = ( 1 - C ) * 255
G = ( 1 - M ) * 255
B = ( 1 - Y ) * 255

	Dim c1 As UInteger
	c1=(R Shl 16)+(G Shl 8)+B
	Return c
End Function